const float gamma = 2.4;
const float inverse_gamma = 1 / gamma;

const vec3 luma_coefficient = vec3(0.2126, 0.7152, 0.0722);

float srgb_to_linear(float x){
    return x > 0.04045 ? pow(x * (1 / 1.055) + 0.0521327, gamma) : x / 12.92;
}

vec3 srgb_to_linear(vec3 x){
    return vec3(srgb_to_linear(x.x),
                srgb_to_linear(x.y),
                srgb_to_linear(x.z));
}

float linear_to_srgb(float x){
    x = max(6.10352e-5, x);
	return min(x * 12.92, pow(max(0.00313067, x), inverse_gamma) * 1.055 - 0.055);
}

vec3 linear_to_srgb(vec3 x){
    return vec3(linear_to_srgb(x.x),
                linear_to_srgb(x.y),
                linear_to_srgb(x.z));
}

float luminance(vec3 x){
	return dot(x, luma_coefficient);
}

vec3 temperature_to_rgb(float temperatureInKelvins){
	vec3 retColor;
	
    temperatureInKelvins = clamp(temperatureInKelvins, 1000, 50000) / 100;
    
    if (temperatureInKelvins <= 66){
        retColor.r = 1;
        retColor.g = saturate(0.39008157876901960784 * log(temperatureInKelvins) - 0.63184144378862745098);
    } else {
    	float t = temperatureInKelvins - 60;
        retColor.r = saturate(1.29293618606274509804 * pow(t, -0.1332047592));
        retColor.g = saturate(1.12989086089529411765 * pow(t, -0.0755148492));
    }
    
    if (temperatureInKelvins >= 66)
        retColor.b = 1;
    else if(temperatureInKelvins <= 19)
        retColor.b = 0;
    else
        retColor.b = saturate(0.54320678911019607843 * log(temperatureInKelvins - 10) - 1.19625408914);

    return srgb_to_linear(retColor);
}   