float remap(float value, float old_low, float old_high, float new_low, float new_high) {
	return  new_low + (value - old_low) * (new_high - new_low) / (old_high - old_low);
}

vec2 sincos(float x){
    return vec2(sin(x), cos(x));
}

vec2 sample_disk(float i, float n, int s){
    float t = (i + n) / float(s); 
    float r = tau * s;

    return sincos(t * r * golden_angle) * t;
}

vec2 sample_disk(vec2 x){
    float t = x.x;
    float r = tau * x.y;

    return sincos(t * r * golden_angle) * t;
}

vec3 sample_disk3(float i, float n, int s){
    float t = (i + n) / float(s); 
    float r = tau * s;

    return vec3(sincos(t * r * golden_angle), sqrt(t));
}


vec3 sample_hemisphere(float x, float s){
    float lon = sqrt(x);
    float lat = tau * (x / s);

    return vec3(cos(lat) * cos(lon), sin(lat), cos(lat) * sin(lon));
}

vec3 sample_hemisphere(vec2 x){
    float lon = sqrt(x.x);
    float lat = tau * x.y;

    return vec3(cos(lat) * cos(lon), sin(lat), cos(lat) * sin(lon));
}

vec2 location_disk(int i, float n, int c){
    float r = (i + n / 6.28) / c;
    float theta = r * 6.28 + n;

    return sincos(theta) * r;
}

vec3 location_disk3(int i, float n, int c){
    float r = (i + n / 6.28) / c;
    float theta = r * 6.28 + n;

    return vec3(sincos(theta), r);
}

vec2 fermats_spiral_n(float i, float t){
    return sincos(i * golden_angle) * sqrt(i / t);
}

vec3 generate_unit_vector(vec2 hash) {
    hash.x *= tau; hash.y = hash.y * 2.0 - 1.0;
    return vec3(vec2(sin(hash.x), cos(hash.x)) * sqrt(1.0 - hash.y * hash.y), hash.y);
}

vec3 generate_cone_vector(vec3 vector, vec2 xy, float angle) {
    vec3 dir = generate_unit_vector(xy);
    float noiseAngle = acos(dot(dir, vector)) * (angle / pi);

    return sin(noiseAngle) * normalize(cross(vector, dir)) + cos(noiseAngle) * vector;
}

vec3 cosine_direction(vec2 coord, vec3 normal){
    float u = rand(78.233 + coord.yx).x;
    float v = rand(10.873 + coord.xy).x;

    float a = 6.2831853 * v;

    float ks = normal.z >= 0 ? 1 : -1;
    float ka = 1 / (1 + abs(normal.z));
    float kb = -ks * normal.x * normal.y * ka;

    vec3 uu = vec3(1 - normal.x * normal.x * ka, ks * kb, -ks * normal.x);
    vec3 vv = vec3(kb, ks - normal.y * normal.y * ka * ks, -normal.y);

    return sqrt(u) * (cos(a) * uu + sin(a) * vv) + sqrt(1 - u) * normal;
}

float filter_gaussian(float x, float sigma){
    const float g = 1 / sqrt(tau * sigma * sigma);
    return (g * exp(-(x * x) / (2.0 * sigma * sigma)));
}

vec2 project_sphere(vec3 dir) {
	float lonlat = atan(-dir.x, -dir.z);
    return vec2(lonlat * (0.5 / 3.16) + 0.5, 0.5 * dir.y + 0.5);
}

vec3 unproject_sphere(vec2 coord) {
	coord *= vec2(tau, pi);

    vec2 lon = sincos(coord.x) * sin(coord.y);
    return vec3(lon.x, 2.0 / 3.16 * coord.y - 1, lon.y);
}

mat3 rotation_mat(vec3 x,vec3 y){
    float d = dot(x,y);
    vec3 cr = cross(y,x);
    
    float s = length(cr);
    
    float id = 1.-d;
    
    vec3 m = cr/s;
    
    vec3 m2 = m*m*id+d;
    vec3 sm = s*m;
    
    vec3 w = (m.xy*id).xxy*m.yzz;
    
    return mat3(
        m2.x,     w.x-sm.z, w.y+sm.y,
        w.x+sm.z, m2.y,     w.z-sm.x, 
        w.y-sm.y, w.z+sm.x, m2.z
    );
}

vec4 to_SH(float value, vec3 dir) {
    const float transferl1 = 0.3849 * pi;
    const float sqrt1Overpi = sqrt(rpi);
    const float sqrt3Overpi = sqrt(rpi * 3.0);

    const vec2 halfnhalf = vec2(0.5, -0.5);
    const vec2 transfer = vec2(pi * sqrt1Overpi, transferl1 * sqrt3Overpi);

    const vec4 foo = halfnhalf.xyxy * transfer.xyyy;

    return foo * vec4(1.0, dir.yzx) * value;
}

vec3 from_SH(vec4 cR, vec4 cG, vec4 cB, vec3 lightDir) {
    const float sqrt1Overpi = sqrt(rpi);
    const float sqrt3Overpi = sqrt(3.0 * rpi);
    const vec2 halfnhalf = vec2(0.5, -0.5);
    const vec2 sqrtOverpi = vec2(sqrt1Overpi, sqrt3Overpi);
    const vec4 foo = halfnhalf.xyxy * sqrtOverpi.xyyy;

    vec4 sh = foo * vec4(1.0, lightDir.yzx);

    return vec3(dot(sh,cR), dot(sh,cG), dot(sh,cB));
}

vec4 texture_bilinear(sampler2D sample_texture, vec2 coord){
    vec2 coord0 = coord + vec2(0, inverse_texel.y);
    vec2 coord1 = coord + vec2(inverse_texel.x, 0);
    vec2 coord2 = coord + vec2(inverse_texel.x, inverse_texel.y);
    vec2 coord3 = coord;

    vec4 s0 = texture(sample_texture, coord0);
    vec4 s1 = texture(sample_texture, coord1);
    vec4 s2 = texture(sample_texture, coord2);
    vec4 s3 = texture(sample_texture, coord3);

    vec2 uv = coord * texel;

    float fu = fract(uv.x);
    float fv = fract(uv.y);

    vec4 tmp0 = mix(s3, s1, fu);
    vec4 tmp1 = mix(s0, s2, fu);

    return mix(tmp0, tmp1, fv);
}

vec4 cubic(float x){
  float x2 = x * x;
  float x3 = x2 * x;

  vec4 w;
       w.x = -x3 + 3 * x2 - 3 * x + 1;
       w.y = 3 * x3 - 6 * x2 + 4;
       w.z = -3 * x3 + 3 * x2 + 3 * x + 1;
       w.w = x3;

  return w / 6.0;
}
//Neuhof
vec4 texture_bicubic(sampler2D sampler, vec2 coord){
  coord *= texel;

  vec2 f = vec2(fract(coord.x), fract(coord.y));

  coord -= f;

  f -= 0.5;

  vec4 cubicX = cubic(f.x);
  vec4 cubicY = cubic(f.y);

  vec4 c = vec4(coord.x - 0.5, coord.x + 1.5, coord.y - 0.5, coord.y + 1.5);
  vec4 s = vec4(cubicX.x + cubicX.y, cubicX.z + cubicX.w, cubicY.z + cubicY.y, cubicY.z + cubicY.w);
  vec4 offset = c + vec4(cubicX.y, cubicX.w, cubicY.y, cubicY.w) / s;

  vec4 tex0 = texture(sampler, vec2(offset.x, offset.z) * inverse_texel);
  vec4 tex1 = texture(sampler, vec2(offset.y, offset.z) * inverse_texel);
  vec4 tex2 = texture(sampler, vec2(offset.x, offset.w) * inverse_texel);
  vec4 tex3 = texture(sampler, vec2(offset.y, offset.w) * inverse_texel);

  float sx = s.x / (s.x + s.y);
  float sy = s.z / (s.z + s.w);

  return mix(mix(tex3, tex2, sx), mix(tex1, tex0, sy), sy);
}