float encode2x8(vec2 x){
    ivec2 bf = ivec2(x * 255);
    return float(bf.x | (bf.y << 8)) / 65535;
}

float encode2x8(float x, float y){
    return encode2x8(vec2(x, y));
}

vec2 decode2x8(float x){
    int bf = int(x * 65535);
    return vec2(bf % 256, bf >> 8) / 255;
}

const vec3 values = exp2(vec3(5, 6, 5));
const vec3 maxValues = values - 1;
const vec3 positions = vec3(1, values.x, values.x * values.y);

float encode3x8(vec3 x){
    #if defined fsh
    //x += (bayer16(gl_FragCoord.xy) - 0.5) / maxValues;
    #endif

    return dot(round(saturate(x) * maxValues), positions) / 65535;
}

vec3 decode3x8(float x){
    return mod(x * (65535 / positions), values) / maxValues;
}

float encodeNormal(vec3 x){
    vec3 b = abs(x);
    vec2 p = x.xy / (b.x + b.y + b.z);

    vec2 normal = x.z <= 0 ? (1 - abs(p.yx)) * sign(p) : p;
         normal = normal * 0.5 + 0.5;

    return encode2x8(normal);
}

vec3 decodeNormal(float x){
    vec2 a = decode2x8(x);
         a = a * 2 - 1;
    vec2 b = abs(a);

    float z = 1 - b.x - b.y;

    return normalize(vec3(z < 0 ? (1 - b.yx) * sign(a) : a, z));
}

vec4 encodeRGBE8(vec3 rgb) {
    float exponentPart = floor(log2(max(max(rgb.r, rgb.g), rgb.b)));
    vec3  mantissaPart = clamp((128.0 / 255.0) * rgb / exp2(exponentPart), 0.0, 1.0);
          exponentPart = clamp((exponentPart + 127.0) / 255.0, 0.0, 1.0);

    return vec4(mantissaPart, exponentPart);
}

vec3 decodeRGBE8(vec4 rgbe) {
    float exponentPart = exp2(rgbe.a * 255.0 - 127.0);
    vec3  mantissaPart = (510.0 / 256.0) * rgbe.rgb;

    return exponentPart * mantissaPart;
}