vec2 WeylNth(int n) {
	return fract(vec2(n * 12664745, n * 9560333) / exp2(24.0));
}

#ifdef TAA  
    vec2 jitter = (WeylNth(frameCounter % 16) * 2.0 - 1.0) * inverse_texel;
#else
    vec2 jitter = vec2(0.0);
#endif

vec3 screen_to_viewspace(vec3 coord){
    coord = coord * 2 - 1;

    vec3 position = proj_mad(gbufferProjectionInverse, coord);
         position = position / (coord.z * gbufferProjectionInverse[2].w + gbufferProjectionInverse[3].w);

    return position;
}

vec3 screen_to_viewspace(vec2 coord, float depth){
    return screen_to_viewspace(vec3(coord, depth));
}

vec3 view_to_worldspace(vec3 position){
    return trans_mad(gbufferModelViewInverse, position);
}

vec3 view_to_screenspace(vec3 coord){
    return (proj_mad(gbufferProjection, coord) / -coord.z) * 0.5 + 0.5;
}

float linear_depth(float depth){
    return (2 * near) / (far + near - depth * (far - near));
}

vec2 screen_velocity(vec3 coord){
    vec3 position = screen_to_viewspace(coord);
    vec3 inversePosition = view_to_worldspace(position);

    vec3 projection = (cameraPosition - previousCameraPosition) + inversePosition;
         projection = trans_mad(gbufferPreviousModelView, projection);
         projection = proj_mad(gbufferPreviousProjection, projection) / -projection.z * 0.5 + 0.5;

    return coord.xy - projection.xy;
}

vec2 projection_velocity(vec3 coord){
    vec3 position = screen_to_viewspace(coord);
    vec3 inversePosition = view_to_worldspace(position);

    vec3 projection = (cameraPosition - previousCameraPosition) + inversePosition;
         projection = trans_mad(gbufferPreviousModelView, projection);
         projection = proj_mad(gbufferPreviousProjection, projection) / -projection.z * 0.5 + 0.5;

    return projection.xy;
}

vec3 terrain_curvature(vec3 position){
    #if ATMOSPHERE_TYPE == 0
        const float planet_radius = 6371e3;
    #elif ATMOSPHERE_TYPE == 1
        const float planet_radius = 1737e3;
    #endif
    
    position.yx *= rotation(position.x / planet_radius);
    position.yz *= rotation(position.z / planet_radius);

    return position;
}

vec3 clouds_curvature(vec3 position){
    const float planet_radius = 6371e3;
    return vec3(position.x, length(position + vec3(0.0, planet_radius, 0.0)) - planet_radius, position.z);
}