#if defined GBUFFERS_TERRAIN
    vec2 wrap_coord(vec2 coord, vec2 size, vec2 position){
        return round((position - coord) / size) * size + coord;
    }

    float normal_depth(vec2 coord, vec2 mid_coord, vec2 size, mat2 tex_grad, float depth){
        return texture_grad(normals, wrap_coord(coord, size, mid_coord), tex_grad[0], tex_grad[1]).w * depth - depth;
    }

    vec2 parallax_occlusion_mapping(vec2 coord, vec3 position, vec3 view_position, vec3 light_position, mat3 tbn, mat2 tex_grad){
        int steps = 64;

        vec2 resolution = tile_info[1];
        vec2 size = tile_info[0];

        float dist = 1.0 - distance(position.xy, position.xy);
        float depth = length(size) * (0.1 + PARALLAX_DEPTH);

        vec3 tangent_vector = normalize(view_position);
        vec3 light_vector = normalize(light_position);

        if(dist > 0.0){
            vec3 ray_increment = inversesqrt(steps) * tangent_vector * dist / -tangent_vector.z;
            vec3 ray_position = vec3(coord, 0.0);

            float height = normal_depth(ray_position.xy, mid_coord, size, tex_grad, depth); 

            for(int i = 0; i < steps && height < ray_position.z; i++){
                ray_position += (ray_position.z - height) * ray_increment;
                height = normal_depth(ray_position.xy, mid_coord, size, tex_grad, depth);
            }

            coord = wrap_coord(ray_position.xy, size, mid_coord);

            vec3 view_space = position + ray_position;
            vec3 screen_space = (diagonal3(gbufferProjection) * view_space + gbufferProjection[3].xyz) / -view_space.z * 0.5 + 0.5;
            
            //gl_FragDepth = screen_space.z;
        }

        return coord;
    }
#endif