#if defined SHADERS_VERTEX
    flat out float material;

    out vec2 coord;
    out vec2 mid_coord;
    out vec2 lightmap;
    out vec3 pom_vector;
    out vec3 pom_light;
    out vec4 color;

    out mat2 tile_info;
    out mat3 position;
    out mat3 tbn;
    out mat3 tbn_pom;

    #include "/program/shaders0/common/animation.glsl"

    void vertex_position(out vec4 gposition, out mat3 position){
        #if defined GBUFFERS_HAND
            mat4 projection = gl_ProjectionMatrix;
        #else
            mat4 projection = gbufferProjection;
        #endif

        position[0] = trans_mad(gl_ModelViewMatrix, layout_position.xyz);

        position[1] = trans_mad(gbufferModelViewInverse, position[0]);
        position[1] = terrain_curvature(position[1]);

        #if defined GBUFFERS_TERRAIN
            position[1] = animation(position[1]);
        #endif
        
        position[2] = position[1] + cameraPosition;

        gposition = trans_mad(gbufferModelView, position[1]).xyzz * diagonal4(projection) + gbufferProjection[3];
        gposition.xy += jitter * gposition.w;
    }   

    void vertex_tbn(out mat3 tbn){
        vec3 n = gl_NormalMatrix * layout_normal.xyz;
             n = mat3(gbufferModelViewInverse) * n;

        vec3 t = gl_NormalMatrix * (layout_tangent.xyz / layout_tangent.w);
             t = mat3(gbufferModelViewInverse) * t;
             
        vec3 b = cross(t, n);

        tbn = mat3(t, b, n);
    }

    void vertex_tile_info(vec2 coord, vec2 mid_coord, vec3 position, mat3 tbn, inout vec3 pom_vector, inout vec3 pom_light, inout mat2 tile_info, inout mat3 tbn_pom){
        tile_info[0] = abs(coord - mid_coord) * 2.0;
        tile_info[1] = vec2(textureSize2D(tex, 0));

        tbn_pom = mat3(tbn[0] * tile_info[1].y / tile_info[1].x, tbn[1], tbn[2]);
        pom_vector = (position * gl_NormalMatrix) * tbn_pom;
        pom_light = (shadowLightPosition * gl_NormalMatrix) * tbn_pom;
    }

    void main(){
        material = layout_material.x;
        coord = layout_texcoord.xy;
        mid_coord = layout_midcoord.xy;
        lightmap = layout_lightmap.xy / 254.0;
        color = layout_color;

        vertex_position(gl_Position, position);
        vertex_tbn(tbn);
        vertex_tile_info(coord, mid_coord, position[0], tbn, pom_vector, pom_light, tile_info, tbn_pom);
    }
#elif defined SHADERS_FRAGMENT
    flat in float material;

    in vec2 coord;
    in vec2 mid_coord;
    in vec2 lightmap;
    in vec3 pom_vector;
    in vec3 pom_light;
    in vec4 color;

    in mat2 tile_info;
    in mat3 position;
    in mat3 tbn;
    in mat3 tbn_pom;

    #include "/program/shaders0/common/parallax.glsl"
    #include "/program/shaders0/common/material.glsl"
    #include "/program/shaders0/common/water_normal.glsl"

    vec3 labpbr_normals(vec2 coord, mat2 tex_grad){
        #if defined GBUFFERS_TERRAIN
            vec3 normal = texture_grad(normals, coord, tex_grad[0], tex_grad[1]).xyz;
        #else
            vec3 normal = texture2(normals, coord).xyz;
        #endif

        #if defined GBUFFERS_TERRAIN && SPEC_FORMAT == 0
            normal.xy = normal.xy * 2.0 - 1.0;
            normal.z = sqrt(1.0 - dot(normal.xy, normal.xy));
        #else   
            normal = normal * 2.0 - 1.0;
        #endif

        return tbn * normal;
    }

    float noise_puddles(vec3 position){
        vec2 scale_position = position.xz * 0.01;
        
        float noise = texture2(noisetex, scale_position).x;
              noise += texture2(noisetex, scale_position * 2).x * 0.5;
              noise += texture2(noisetex, scale_position * 7).x * 0.25;

        return saturate(noise / 1.1);
    }

    float rain_puddles(vec3 position, vec3 normal, float lightmap, float porosity){
        lightmap = saturate(lightmap * 1.05);
        lightmap = pow2(lightmap);

        float puddles = noise_puddles(position);
              puddles = wetness < 0.01 ? 0.0 : puddles * lightmap;
              puddles *= saturate(dot(normal, vec3(0.0, 1.0, 0.0)));
              puddles *= wetness;

        return puddles * (1.0 - pow2(porosity));
    }

    void main(){
        mat2 tex_grad = mat2(dFdx(coord), dFdy(coord));

        #if defined GBUFFERS_TERRAIN
            #ifdef PARALLAX
                vec2 uv = parallax_occlusion_mapping(coord, position[0], pom_vector, pom_light, tbn_pom, tex_grad);
            #else
                vec2 uv = coord;
            #endif

            t.textures0 = texture_grad(tex, uv, tex_grad[0], tex_grad[1]);
            t.textures2 = texture_grad(specular, uv, tex_grad[0], tex_grad[1]);
        #else
            vec2 uv = coord;

            t.textures0 = texture2(tex, uv);
            t.textures2 = texture2(specular, uv);
        #endif

        float ID = 1.0, roughness, f0, porosity, emissive;
        materials(ID, material);
        specular_material(t.textures2, ID, roughness, f0, porosity, emissive);

        bool water = ID == 2.0;

        float noise = blue_noise(gl_FragCoord.xy).a / 254.0;
        float lightmap0 = saturate(lightmap.x + noise);
        float lightmap1 = saturate(lightmap.y + noise);

        #if defined GBUFFERS_TERRAIN || defined GBUFFERS_ENTITY
            float puddles = rain_puddles(position[2], tbn[2], lightmap1, porosity);

            roughness = mix(roughness, 0.021, puddles);
            f0 = mix(f0, 0.20, puddles);
        #endif

        #if defined GBUFFERS_TRANSLUCENT
            if(!gl_FrontFacing) discard;
        #endif

        #if !defined GBUFFERS_BASIC && !defined GBUFFERS_ENCHANT && !defined GBUFFERS_TRANSLUCENT
            if(t.textures0.w < 0.128) discard;
        #endif

        #ifdef MC_NORMAL_MAP
            #if defined GBUFFERS_TERRAIN
                lightmap1 *= sqrt(texture_grad(normals, uv, tex_grad[0], tex_grad[1]).z);
            #endif

            t.textures1.xyz = labpbr_normals(uv, tex_grad);
        #else
            t.textures1.xyz = tbn[2];
        #endif 

        t.textures0 = t.textures0 * color;

        #ifndef ALPHA_TEXTURES
            #if defined GBUFFERS_TERRAIN
                t.textures0.xyz = vec3(1.0);
            #endif
        #endif

        #if defined GBUFFERS_ENTITY
            t.textures0.xyz = mix(t.textures0.xyz, entityColor.xyz, entityColor.w);
        #endif

        #if defined GBUFFERS_TRANSLUCENT
            t.textures0 = water ? vec4(0.0, 0.0, 0.0, 0.0) : t.textures0;
            t.textures1.xyz = water ? tbn * water_normal(position[1], position[2], water) : t.textures1.xyz;
        #endif

        #if defined GBUFFERS_ENCHANT
            buffers0.xyz = t.textures0.xyz;
            buffers0.w = 1.0;
            /* RENDERTARGETS:1 */
        #elif defined GBUFFERS_WEATHER
            buffers0.xyz = texture2(colortex15, coord).xyz;
            buffers0.w = t.textures0.w;
            /* RENDERTARGETS:15 */
        #else
            buffers0.xyz = t.textures0.xyz;

            #if defined GBUFFERS_TRANSLUCENT
                buffers0.w = t.textures0.w;
            #else
                buffers0.w = 1.0;
            #endif

            buffers1.x = encodeNormal(t.textures1.xyz);
            buffers1.y = encode2x8(lightmap0, lightmap1);
            buffers1.z = encode2x8(roughness, f0);
            buffers1.w = encode2x8(emissive, ID / 255.0);

            /* RENDERTARGETS:1,2 */
        #endif
    }
#endif  