vec3 planet_sphere(vec3 position){
        atmosphere_constant ac = atmosphere();

        const float delta = 0.05;
        const float radius = 58232e3;
        const float dist = 1443.53e7;
        const float angular_radius = 2.0 * radius / dist;

        vec3 transmittance = vec3(1.0);

        vec3 light_direction = mat3(gbufferModelViewInverse) * normalize(shadowLightPosition);
        vec3 sun_direction = mat3(gbufferModelViewInverse) * normalize(sunPosition);

        vec3 planet_direction = normalize(vec3(-0.5, 0.7, -0.6));
             planet_direction = planet_direction * rotation_mat(vec3(1.0, 0.0, 0.0), sun_direction);

        vec3 view_position = vec3(0.0, angular_radius, 0.0);
        vec2 sphere = ray_sphere_intersection(-planet_direction, position, angular_radius);

        vec3 normal = normalize(position * sphere.x - planet_direction);

        float n0 = fbm_3(vec3(normal.x + delta, normal.y, normal.z), vec3(0.0), 5);
        float n1 = fbm_3(vec3(normal.x, normal.y + delta, normal.z), vec3(0.0), 5);
        float n2 = fbm_3(vec3(normal.x, normal.y, normal.z + delta), vec3(0.0), 5);

        normal = normalize(vec3(normal.x * n0, normal.y * n1, normal.z * n2));

        vec3 H = normalize(light_direction - position);

        float NdotL = dot(normal, sunAngle < 0.5 ? light_direction : -light_direction);
        float NdotV = dot(normal, -position);
        float LdotH = dot(light_direction, H);

        float direct_diffuse = saturate(NdotL) * disney_diffuse(NdotL, NdotV, LdotH, 0.343, 1.0);

        vec2 pos = vec2(normal.x * 0.7, normal.z * 0.1) * 0.14;
        float noise = fbm_2(pos, vec2(0.0), 6);

        vec3 color0 = vec3(0.101961,0.619608,0.666667) * 0.5;
        vec3 color1 = vec3(0.666667,0.666667,0.498039);
        vec3 color2 = vec3(0.666667,1,1);

        vec3 albedo = mix(color0, color1, pow6(1.0 - noise * 0.5 + 0.4));
             albedo = mix(albedo, color2, pow2(1.0 - noise * 0.5 + 0.2));
             albedo = srgb_to_linear(albedo);

        vec3 sunlight = atmosphere_transmittance(ac, light_direction, light_direction) * ac.sun_luminance;
        vec3 skylight = atmospheric_scattering(vec3(0.0, 1.0, 0.0), light_direction, -light_direction, transmittance);

        return sphere.y < 0.0 ? vec3(0.0) : albedo * direct_diffuse * sunlight;
    }