#include "/program/shaders1/common/shadow/shadow.glsl"

#include "/program/shaders1/common/indirect/skylight.glsl"
#include "/program/shaders1/common/indirect/filter.glsl"

#include "/program/shaders1/common/volumetric/clouds_3D.glsl"

vec3 emissive(materials m, vec3 albedo, vec3 blocklight, float emissive){
     if(m.torch){
          albedo *= pow3(length(albedo)) * blocklight;
     }

     if(m.glowstone || m.lava || m.fire || m.redstone){
          albedo *= blocklight;
     }

     return albedo * pi * emissive;
}

float held_light(vec3 position, float NdotV, bool hand){
     float ID = heldItemId == 1001 || heldItemId == 1002 || heldItemId == 1003 || heldItemId == 1004 || heldItemId == 1005 || heldItemId == 1006 ? 1 : 0;

     float dist = pow2(length(position)) + 0.75;
           dist = 1.0 / dist;

     if(hand){
          NdotV = 1.0;
     }

     return dist * ID * max(1e-7, NdotV);
}

vec3 diffuse_light(gbuffers_data g, space_position sp, materials m, vec2 coord, vec3 albedo, vec3 normal, vec3 view_position, vec3 position, vec3 inverse_position, vec4 noise){
     const float emissive_brightness = 800.0;

     #if defined VOLUMETRIC_CLOUD && defined CLOUDS_SHADOW
          float rain_brightness = 1.0;
     #else
          float rain_brightness = mix(1.0, 0.0, wetness);
     #endif

     g.roughness = pow2(g.roughness);

     g.lightmap.x = saturate(g.lightmap.x);
     g.lightmap.x = pow(g.lightmap.x, 1.0 / 0.5);
     g.lightmap.x = pow3(g.lightmap.x);

     if(!m.water && isEyeInWater == 0){
          g.lightmap.y = saturate(g.lightmap.y);
          g.lightmap.y = 1.0 - pow(1.0 - g.lightmap.y, 0.75);
          g.lightmap.y = pow2(g.lightmap.y);
     }

     vec3 vector = f_normalize(position);
     vec3 inverse_vector = f_normalize(inverse_position);

     vec3 H = normalize(sp.light_vector - inverse_vector);

     float NdotL = dot(normal, sp.light_vector);
     float NdotV = max(1e-7, dot(normal, -inverse_vector));
     float LdotH = dot(sp.light_vector, H);

     float direct_diffuse = saturate(NdotL) * disney_diffuse(NdotL, NdotV, LdotH, g.roughness, m.ior);
     float indirect_diffuse = disney_diffuse(NdotV, NdotV, NdotV, g.roughness, m.ior);
     float ggx_diffuse = ggx_hightligh(m, g.normal, inverse_vector, sp.light_vector, g.roughness, g.f0);

     float clouds_shadow = clouds_3D_shadow(inverse_position);
     float contact_shadow = m.hand ? 1.0 : screenspace_contact_shadow(m, view_position, position, mat3(gbufferModelView) * sp.light_vector);
     float ao = m.hand ? 1.0 : ao_filter(coord / INDIRECT_LOD, 4.0);
     
     vec3 sunlight = transmittance_color(sp.sun_vector, sp.moon_vector) * rain_brightness;
     vec3 skylight = skylight(g.normal, contact_shadow);
     vec3 blocklight = temperature_to_rgb(BLOCKLIGHT_TEMPERATURE);

     vec3 fresnel0 = 1.0 - fresnel(saturate(LdotH), m.ior, g.f0, albedo, m.metal);
     vec3 fresnel1 = (1.0 - g.f0) + g.albedo;

     vec3 ggx_albedo = g.f0 > 225.9 / 255.0 || m.translucent && !m.water ? albedo : vec3(1.0);

     vec3 shadow = shadow(m, inverse_position, g.normal, NdotL);
          shadow *= contact_shadow * clouds_shadow;

     vec3 rsm = rsm_filter(coord / INDIRECT_LOD, 4.0);

     if(m.grass || m.leaves || m.fire){
          direct_diffuse = saturate((NdotV + 0.5) / 2.25) * rpi; 
     }

     if(m.textured){
          direct_diffuse = 0.5 * rpi;
          indirect_diffuse = 1.5;
     }

     #if defined OVER_WORLD || defined END_WORLD
          vec3 color = shadow * direct_diffuse * sunlight * fresnel0;
               color += g.lightmap.y * ao * indirect_diffuse * skylight * fresnel1;
               color += g.lightmap.x * blocklight * emissive_brightness * fresnel1;
               color += held_light(inverse_position, NdotV, m.hand) * blocklight * emissive_brightness * fresnel1;
               color += emissive(m, albedo, blocklight, g.emissive) * emissive_brightness;

          #if defined DEFERRED6
               color += clouds_shadow * rsm * fresnel1;
          #endif

               color *= albedo;
          
          if(!m.textured && !m.basic){
               color += shadow * ggx_albedo * ggx_diffuse * sunlight;
          }
     #elif defined NETHER_WORLD
          vec3 color = albedo * saturate(dot(g.normal, vec3(0.0, -1.0, 0.0))) * 0.75;
               color += albedo * 1.1;
               color += albedo * g.lightmap.x * blocklight * emissive_brightness * fresnel1;
               color += albedo * held_light(inverse_position, NdotV, m.hand) * blocklight * emissive_brightness * fresnel1;
               color += albedo * emissive(m, albedo, blocklight, g.emissive) * emissive_brightness;
     #endif

     return color;
}