vec3 soft_shadow(shadow_settings s, materials m, water_settings ws, vec3 shadow_position, vec3 shadow_normal, float bias){
    vec3 shadow = vec3(0);

    float noise = blue_noise(gl_FragCoord.xy).w;

    #if SHADOW_TYPE == 2
        vec2 radius_offset = pcss(s, shadow_position, bias, noise);
             radius_offset = m.leaves ? vec2(0.005) : radius_offset;
    #else
        vec2 radius_offset = vec2(s.inverse_resolution) * 0.75;
             radius_offset = m.leaves ? vec2(0.005) : radius_offset;
    #endif

    for(int i = 0; i < s.steps0; i++){
        vec2 offset = sample_disk(i, noise, s.steps0);
        
        vec3 shadow_position0 = vec3(offset, -bias) * radius_offset.x + shadow_position;
             shadow_position0 = shadow_distortion(s, shadow_position0) * 0.5 + 0.5;

        vec3 shadow_position1 = vec3(offset, -bias) * radius_offset.y + shadow_position;
             shadow_position1 = shadow_distortion(s, shadow_position1) * 0.5 + 0.5;

        vec4 color0 = texture2(shadowcolor0, shadow_position0.xy);
        vec4 color1 = texture2(shadowcolor1, shadow_position1.xy);

        vec3 s_normal = decodeNormal(color1.x * 2.0 - 1.0);
        float ID = color1.w * 255.0;

        float depth0 = texture2(shadowtex0, shadow_position0.xy).x;
        float depth1 = texture2(shadowtex1, shadow_position1.xy).x;

        //bias = (depth0 == depth1) ? (dot(shadow_normal, s_normal) > 0.01 ? bias : 0.0) : bias;

        float shadow_depth0 = depth0 > shadow_position0.z - bias ? 1.0 : 0.0;
        float shadow_depth1 = depth1 > shadow_position1.z - bias ? 1.0 : 0.0;

        color0.xyz = srgb_to_linear(color0.xyz);
        color0.w = (1.0 - color0.w);
            
        float translucent_depth0 = (depth0 * 8.0 - 4.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;
        float translucent_depth1 = (depth1 * 8.0 - 4.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;

        float depth = translucent_depth1 - translucent_depth0;
        float caustics_depth = saturate(depth * 2.0);

        #ifdef WATER_CAUSTICS
            float caustics = color1.z * 2.0 - 1.0;
                  caustics = pow(caustics * 8.0, WATER_CAUSTICS_STRENGTH);
                  caustics = caustics + caustics_depth * (1.0 - caustics_depth);
        #else
            float caustics = 1.0;
        #endif

        vec3 water_color = exp2(ws.attenuation_coefficient * 0.18 * depth) * caustics;

        if(ID == 2){
            shadow += mix(vec3(shadow_depth0), water_color, saturate(shadow_depth1 - shadow_depth0));
        } else {
            #ifdef COLORED_SHADOW
                shadow += mix(vec3(shadow_depth0), color0.xyz * color0.w, saturate(shadow_depth1 - shadow_depth0));
            #else
                shadow += vec3(shadow_depth1);
            #endif
        }
    }

    return shadow / s.steps0;
}