bool screenspace_raytrace(vec3 view_position, vec3 position, vec3 direction, float NdotV, float noise, float z_treshold, int steps, inout vec3 hit_position){   
    direction = abs(-position.z) * direction;
    direction = normalize(view_to_screenspace(direction + position) - view_position);

    bool intersect = false;

    float max_length = 1.0 / float(steps);
    float min_length = max_length * 0.01;

    float ray_length = mix(min_length, max_length, NdotV);
    float ray_weight = 1.0 / abs(direction.z);

    vec3 sample_increment = direction * ray_length;
    vec3 sample_position = view_position + sample_increment * (noise + 0.1);

    float depth = texture(depthtex1, sample_position.xy).x;

    for(int i = 0; i < steps; i++){
        if(saturate(sample_position) != sample_position) break;

        if(depth < sample_position.z){
            depth = texture(depthtex1, sample_position.xy).x;

            float depth0 = linear_depth(sample_position.z);
            float depth1 = linear_depth(depth);

            float dist = abs(depth1 - depth0) / depth0;

            if(dist < z_treshold && depth0 > -far){
                intersect = true;
                break;
            }
        }

        ray_length = clamp(abs(depth - sample_position.z) * ray_weight, min_length, max_length);
        sample_position += direction * ray_length;
        depth = texture(depthtex1, sample_position.xy).x;
    }

    if(intersect){
        sample_position -= direction * ray_length * 0.5;
        depth = texture(depthtex1, sample_position.xy).x;

        for(int i = 0; i < 4; i++){
            if(saturate(sample_position) != sample_position) break;

            if(depth < sample_position.z){
                depth = texture(depthtex1, sample_position.xy).x;

                float depth0 = linear_depth(sample_position.z);
                float depth1 = linear_depth(depth);

                float dist = abs(depth1 - depth0) / depth0;

                if(dist < z_treshold && depth0 > -far){
                    break;
                }
            }
        }

        ray_length = clamp(abs(depth - sample_position.z) * ray_weight, min_length, max_length);
        sample_position += direction * ray_length;
        depth = texture(depthtex1, sample_position.xy).x;
    }

    hit_position = vec3(sample_position.xy, depth);

    if(depth >= 1.0) intersect = false;

    return intersect;
}