    vec3 volumetric_water_fog(vec3 position0, vec3 position1, vec3 direction, vec3 sun_position, vec3 moon_position, float noise, float lightmap, const int steps, inout vec3 transmittance){
        water_settings ws = swater();

        vec3 direct, indirect;

        float VdotL = dot(normalize(position1), direction);
        float phase = hg_phase(VdotL, ws.mie_g);
            
        vec3 ray_increment = (position1 - position0) / steps;
        vec3 ray_position = ray_increment * noise + position0;

        vec3 shadow_position0 = shadow_position(position0);
        vec3 shadow_position1 = shadow_position(position1);

        vec3 shadow_increment = (shadow_position1 - shadow_position0) / steps;
        vec3 shadow_position = shadow_increment * noise + shadow_position0;
        
        float ray_length = length(ray_increment);

        vec3 density = ws.attenuation_coefficient * ws.density * ray_length;
        vec3 step_transmittance = exp2(-density);

        #ifdef WATER_FOG
            for(int i = 0; i < steps; i++, ray_position += ray_increment, shadow_position += shadow_increment){
                if(max3(transmittance) <= ws.treshold) break;

                vec3 step_transmittance_fraction = 1.0 - step_transmittance;
                vec3 sample_transmittance = transmittance * step_transmittance_fraction;

                vec3 shadow = volumetric_shadow(ray_position, shadow_position, true);

                direct += sample_transmittance * shadow;
                indirect += sample_transmittance;

                transmittance *= step_transmittance;
            }

            vec3 sunlight = transmittance_color(sun_position, moon_position);
            vec3 skylight = texture2(colortex8, project_sphere(vec3(0.0, 1.0, 0.0)) / 2.0).xyz;
            vec3 albedo = ws.scattering_coefficient * ws.density * 1.2;

            skylight = skylight + skylight.g;

            vec3 scattering = direct * sunlight * phase;
                scattering += indirect * skylight;
                scattering *= albedo * saturate(1.0 - transmittance);
            
            return scattering;
        #else
            return exp(-ws.attenuation_coefficient * ws.density * distance(position1, position0));
        #endif
    }