#if defined SHADERS_VERTEX
    out vec2 coord;

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);
    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;

    #include "/program/shaders1/common/atmosphere/atmosphere.glsl"
    #include "/program/shaders1/common/volumetric/clouds_3D.glsl"
    
    float clouds_shadowmap(vec2 coord, vec3 direction){
        atmosphere_constant ac = atmosphere();
        clouds_3D_settings vc = s_clouds3D();

        const int steps = CLOUDS_SHADOW_QUALITY;

        vec2 resolution = texel / 1024.0;
        vec2 shadow_coord = coord * resolution;

        float transmittance = 0.0;
        float fade = smoothstep(0.125, 0.025, direction.y);
        float noise = blue_noise(gl_FragCoord.xy).w;

        if(saturate(shadow_coord) == shadow_coord){
            vec3 shadow_position = vec3(saturate(shadow_coord), 0.0) * 2.0 - 1.0;
                 shadow_position.xy *= 200.0;
                 shadow_position = mat3(shadowModelViewInverse) * shadow_position;
                 
                 shadow_position += cameraPosition;
                 shadow_position += direction * (vc.altitude_min - shadow_position.y) / direction.y;

            vec3 view_position = shadow_position + vec3(0.0, ac.planet_radius, 0.0);

            vec2 sphere_min = ray_sphere_intersection(view_position, direction, ac.planet_radius + vc.altitude_min);
            vec2 sphere_max = ray_sphere_intersection(view_position, direction, ac.planet_radius + vc.altitude_max);

            float dist0 = eyeAltitude > vc.altitude_max ? sphere_max.x : sphere_min.y;
            float dist1 = eyeAltitude > vc.altitude_max ? sphere_min.x : sphere_max.y;

            float s_length = (dist1 - dist0) / steps;

            vec3 ray_increment = direction * s_length;
            vec3 ray_position = direction * (dist0 + s_length * 0.5) + shadow_position;

            for(int i = 0; i < steps; i++, ray_position += ray_increment){
                transmittance += clouds_3D_density(vc, ray_position);
            }

            #if defined VOLUMETRIC_CLOUD && defined CLOUDS_SHADOW && defined OVER_WORLD
                return exp2(-transmittance * s_length) * (1.0 - fade) + fade;
            #else
                return 1.0;
            #endif
        }
    }

    void main(){
        vec2 scale_coord = coord * ATMOSPHERE_LOD;

        if(saturate(scale_coord) == scale_coord){
            gbuffers_data g = gbuffers(scale_coord);
            space_position sp = position(scale_coord, g.depth, false);
            materials m = material(g.depth, g.ID);

            vec3 projection = unproject_sphere(scale_coord);
                 projection = normalize(projection);

            vec3 transmittance = vec3(1.0);

            buffers0.xyz = atmospheric_scattering(projection, sp.sun_vector, sp.moon_vector, transmittance);
            buffers1.xyz = transmittance;
        }

        buffers0.w = clouds_shadowmap(coord, mat3(gbufferModelViewInverse) * normalize(shadowLightPosition));
        buffers1.w = 1.0;

        /* RENDERTARGETS:8,7 */
    }
#endif