#if defined SHADERS_VERTEX
    out vec2 coord;

    void main(){
        coord = layout_texcoord.xy;

        gl_Position = vec4(coord * 2.0 - 1.0, 0.0, 1.0);
    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;

    #include "/program/shaders1/common/atmosphere/atmosphere.glsl"

    #include "/program/shaders1/common/volumetric/clouds_2D.glsl"
    #include "/program/shaders1/common/volumetric/clouds_3D.glsl"

    vec4 sky(vec3 position, vec3 direction, vec3 sun_direction, vec3 moon_direction, float noise, const int steps_3D){
        atmosphere_constant ac = atmosphere();

        clouds_3D_settings vc = s_clouds3D();
        clouds_2D_settings pc = s_clouds2D();

        vec3 transmittance = vec3(1.0);

        float clouds_transmittance = 1.0;

        vec3 atmosphere = atmospheric_scattering(position, sun_direction, moon_direction, transmittance) * 0.85;
        vec3 rain_atmosphere = mix(atmosphere, vec3(luminance(atmosphere)), wetness);
        vec3 attenuation = ac.wave_lambda_rayleigh + ac.wave_lambda_mie;

        #if !defined VOLUMETRIC_CLOUD && !defined CLOUDS_SHADOW
            atmosphere = mix(atmosphere, vec3(luminance(atmosphere)), wetness);
        #endif
        
        #if ATMOSPHERE_TYPE == 0
            #if defined OVER_WORLD
                #ifdef PLANAR_CLOUD
                    vec3 dist0 = vec3(0.0);
                    vec4 clouds_2D = clouds_2D(position, direction, sun_direction, moon_direction, dist0);
                    
                    atmosphere = mix(atmosphere * clouds_2D.w + clouds_2D.xyz, rain_atmosphere, 1.0 - exp2(-length(dist0) * max3(attenuation)));
                    clouds_transmittance *= clouds_2D.w;
                #endif

                #ifdef VOLUMETRIC_CLOUD
                    vec3 dist1 = vec3(0.0);
                    vec4 clouds_3D = clouds_3D(position, direction, sun_direction, moon_direction, noise, steps_3D, dist1);

                    atmosphere = mix(atmosphere * clouds_3D.w + clouds_3D.xyz, rain_atmosphere, 1.0 - exp2(-length(dist1) * max3(attenuation)));
                    clouds_transmittance *= clouds_3D.w;
                #endif
            #endif
            
            return vec4(atmosphere, clouds_transmittance);
        #elif ATMOSPHERE_TYPE == 1
            return vec4(0.0, 0.0, 0.0, 1.0);
        #endif
    }

    void main(){
        atmosphere_constant ac = atmosphere();

        clouds_3D_settings vc = s_clouds3D();
        clouds_2D_settings pc = s_clouds2D();

        vec2 scale_coord = coord * ATMOSPHERE_LOD;

        if(saturate(scale_coord) == scale_coord){
            gbuffers_data g = gbuffers(scale_coord);
            space_position sp = position(scale_coord, g.depth, false);
            materials m = material(g.depth, g.ID);

            vec2 offset = scale_coord * texel / ATMOSPHERE_LOD;
            vec4 noise = blue_noise(offset + jitter * 0.5);

            vec3 projection = unproject_sphere(scale_coord);
                 projection = normalize(projection);

            buffers0 = sky(sp.inverse_vector1, sp.light_vector, sp.sun_vector, sp.moon_vector, noise.w, vc.steps0);
            buffers1 = sky(projection, sp.light_vector, sp.sun_vector, sp.moon_vector, 1.0, 4);
        }
        
        /* RENDERTARGETS:6,9 */
    }
#endif