#if defined SHADERS_VERTEX
    out vec2 coord;

    void vertex_position(vec2 coord, out vec4 gposition){
        #ifdef UPSIDE_DOWN_WORLD
            gposition.xy = coord * -2.0 + 1.0;
        #else
            gposition.xy = coord * 2.0 - 1.0;
        #endif

        gposition.z = 0.0;
        gposition.w = 1.0;
    }

    void main(){
        coord = layout_texcoord.xy;

        vertex_position(coord, gl_Position);
    }
#elif defined SHADERS_FRAGMENT
    in vec2 coord;  

    #include "/program/shaders2/common/taa.glsl"

    vec3 wide_screen(vec3 color){
        #if WIDE_SCREEN == 1
            const float screen0 = 0.9 - WIDE_SCREEN_SIZE;
            const float screen1 = 0.1 + WIDE_SCREEN_SIZE;

            return coord.y > screen0 || coord.y < screen1 ? vec3(0) : color;
        #elif WIDE_SCREEN == 2

            const float screen0 = 0.69 - WIDE_SCREEN_SIZE;
            const float screen1 = 0.31 + WIDE_SCREEN_SIZE;

            return coord.x > screen0 || coord.x < screen1 ? vec3(0) : color;
        #elif WIDE_SCREEN == 0
            return color;
        #endif
    }

    vec3 damage_visual(vec3 color){
        #ifdef DAMAGE_VISUAL
            return color + (vec3(0.7, 0, 0) * damage);
        #else
            return color;
        #endif
    }

    vec3 vignette(vec3 color){
        float dist = pow2(distance(coord, vec2(0.5))) * (1.2 + VIGNETTE_SIZE);

        #ifdef VIGNETTE
            return color * (1.0 - dist);
        #else
            return color;
        #endif
    }

    void barrel_distortion(inout vec2 coord){
        coord -= 0.5;

        float theta = atan(coord.x, coord.y);
        float radius = sqrt(dot(coord, coord));
              radius -= 0.11 * pow4(radius);

        coord = 0.5 + radius * vec2(sin(theta), cos(theta));
    }

    void nether_heatwaves(inout vec2 coord){
        float depth = texture2(depthtex1, coord).x;
        
        vec2 offset = (fract(coord / noiseTextureResolution) * noiseTextureResolution) + sin(frameTimeCounter * 0.02);
        vec3 position = screen_to_viewspace(coord, depth);

        float dist = 1.0 - exp(-length(position) * 0.00022);

        #if defined NETHER_WORLD
            #ifdef NETHER_HEATWAVES
                coord.x += (texture2(noisetex, offset).x / 20.0) * dist;
                coord.y += (texture2(noisetex, offset).y / 20.0) * dist;
            #endif
        #endif
    }

    vec3 texture_sharpen(sampler2D tex, vec2 coord, vec2 t){
        temporal_settings ts = temporal_s();

        vec3 color = texture2(tex, coord).xyz;
        //vec3 color = texel_fetch(tex, ivec2(coord * texel)).xyz;
        //vec3 color = smaa_filter(tex, coord, texel);

        vec4 color_avg  = vec4(texture2(tex, coord + vec2(t.x, 0.0)).xyz, 1.0);
             color_avg += vec4(texture2(tex, coord + vec2(-t.x, 0.0)).xyz, 1.0);
             color_avg += vec4(texture2(tex, coord + vec2(0.0, t.y)).xyz, 1.0);
             color_avg += vec4(texture2(tex, coord + vec2(0.0, -t.y)).xyz, 1.0);

             color_avg += vec4(texture2(tex, coord + vec2(t.x, t.y)).xyz, 1.0);
             color_avg += vec4(texture2(tex, coord + vec2(-t.x, t.y)).xyz, 1.0);
             color_avg += vec4(texture2(tex, coord + vec2(t.x, -t.y)).xyz, 1.0);
             color_avg += vec4(texture2(tex, coord + vec2(-t.x, -t.y)).xyz, 1.0);
            
             color_avg.xyz /= color_avg.w;

        float contrast = 1.0 - luminance(color) / color_avg.w;
              contrast = saturate(contrast);

        #ifdef ADAPTIVE_SHARPEN
            color *= 1.0 + ts.sharpen * contrast;
            color -= color_avg.xyz * ts.sharpen * contrast;
        #endif

        return color;
    }

    vec3 night_vision(vec2 coord, vec3 color){
        vec3 noise = rand3(coord + sin(frameTimeCounter));
        
        float intensity = luminance(color);
              intensity = saturate(0.5 * (intensity - 0.5) + 0.5);

        vec3 green_color = vec3(0.0, intensity * 2.0, 0.0);

        #ifdef GREEN_NV
            return mix(color, (color + (noise * 0.2)) * green_color, nightVision);
        #else
            return color;
        #endif
    }

    vec3 blindness_effect(vec2 coord, vec3 color){
        float depth = texture2(depthtex0, coord).x;

        vec3 position = screen_to_viewspace(coord, depth);
        vec3 inverse_position = view_to_worldspace(position);

        float dist = 1.0 - exp2(-length(inverse_position) * 0.25 * blindness);

        return mix(color, vec3(0.0), dist);
    }

    void main(){
        vec2 uv = coord;

        barrel_distortion(uv);
        //nether_heatwaves(uv);

        vec3 color = texture_sharpen(colortex0, uv, inverse_texel);
             color = damage_visual(color);
             color = blindness_effect(uv, color);
             color = night_vision(uv, color);
             color = wide_screen(color);
             color = vignette(color);
             color += blue_noise(gl_FragCoord.xy).w / 254.0;

        gl_FragColor = vec4(color, 1.0);
    }
#endif  